"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AggregateRoot = void 0;
class AggregateRoot {
    constructor() {
        this._domainEvents = [];
    }
    get domainEvents() {
        return this._domainEvents;
    }
    addEvent(domainEvent) {
        this._domainEvents.push(domainEvent);
    }
    clearEvents() {
        this._domainEvents = [];
    }
    publishEvents(notifier) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all(this.domainEvents.map((event) => __awaiter(this, void 0, void 0, function* () {
                return notifier.notifyEvent(event.name, event.payload);
            })));
            this.clearEvents();
            for (const attr in this) {
                const attribute = this[attr];
                if (attribute instanceof AggregateRoot) {
                    yield attribute.publishEvents(notifier);
                }
                else if (attribute instanceof Array) {
                    const prop = attribute;
                    for (const nestedAttribute of prop) {
                        if (nestedAttribute instanceof AggregateRoot) {
                            yield nestedAttribute.publishEvents(notifier);
                        }
                    }
                }
            }
        });
    }
}
exports.AggregateRoot = AggregateRoot;
//# sourceMappingURL=aggregate-root.js.map